==> pytype-2018.9.26.tar.gz: pytype-2018.9.26/pytype/abc_overlay.py <==
"""Implementation of special members of Python 2's abc library."""

from pytype import abstract
from pytype import overlay
from pytype import special_builtins


class ABCOverlay(overlay.Overlay):
  """A custom overlay for the 'abc' module."""

  def __init__(self, vm):
    member_map = {
        "abstractmethod": AbstractMethod,
        "abstractproperty": AbstractProperty
    }
    ast = vm.loader.import_name("abc")
    super(ABCOverlay, self).__init__(vm, "abc", member_map, ast)


class AbstractMethod(abstract.PyTDFunction):
  """Implements the @abc.abstractmethod decorator."""

  def __init__(self, name, vm):
    super(AbstractMethod, self).__init__(
        *abstract.PyTDFunction.get_constructor_args(name, vm, "abc"))

  def call(self, node, unused_func, args):
    """Marks that the given function is abstract."""
    self.match_args(node, args)

    # Since we have only 1 argument, it's easy enough to extract.
    if args.posargs:
      func_var = args.posargs[0]
    else:
      func_var = args.namedargs["function"]

    for func in func_var.data:
      if isinstance(func, (abstract.Function, abstract.BoundFunction)):
        func.is_abstract = True

    return node, func_var


class AbstractProperty(special_builtins.PropertyTemplate):
  """Implements the @abc.abstractproperty decorator."""

  def __init__(self, name, vm):
    super(AbstractProperty, self).__init__(vm, name, "abc")

  def call(self, node, funcv, args):
    property_args = self._get_args(args)
    for v in property_args.values():
      for b in v.bindings:
        f = b.data
        # If this check fails, we will raise a 'property object is not callable'
        # error down the line.
        # TODO(mdemello): This is in line with what python does, but we could
        # have a more precise error message that insisted f was a class method.
        if isinstance(f, abstract.Function):
          f.is_abstract = True
    return node, special_builtins.PropertyInstance(
        self.vm, self.name, self, **property_args).to_variable(node)

==> pytype-2019.4.12.tar.gz: pytype-2019.4.12/pytype/abc_overlay.py <==
"""Implementation of special members of Python 2's abc library."""

from pytype import abstract
from pytype import overlay
from pytype import special_builtins


class ABCOverlay(overlay.Overlay):
  """A custom overlay for the 'abc' module."""

  def __init__(self, vm):
    member_map = {
        "abstractmethod": AbstractMethod.make,
        "abstractproperty": AbstractProperty
    }
    ast = vm.loader.import_name("abc")
    super(ABCOverlay, self).__init__(vm, "abc", member_map, ast)


class AbstractMethod(abstract.PyTDFunction):
  """Implements the @abc.abstractmethod decorator."""

  @classmethod
  def make(cls, name, vm):
    return super(AbstractMethod, cls).make(name, vm, "abc")

  def call(self, node, unused_func, args):
    """Marks that the given function is abstract."""
    self.match_args(node, args)

    # Since we have only 1 argument, it's easy enough to extract.
    if args.posargs:
      func_var = args.posargs[0]
    else:
      func_var = args.namedargs["function"]

    for func in func_var.data:
      if isinstance(func, (abstract.Function, abstract.BoundFunction)):
        func.is_abstract = True

    return node, func_var


class AbstractProperty(special_builtins.PropertyTemplate):
  """Implements the @abc.abstractproperty decorator."""

  def __init__(self, name, vm):
    super(AbstractProperty, self).__init__(vm, name, "abc")

  def call(self, node, funcv, args):
    property_args = self._get_args(args)
    for v in property_args.values():
      for b in v.bindings:
        f = b.data
        # If this check fails, we will raise a 'property object is not callable'
        # error down the line.
        # TODO(mdemello): This is in line with what python does, but we could
        # have a more precise error message that insisted f was a class method.
        if isinstance(f, abstract.Function):
          f.is_abstract = True
    return node, special_builtins.PropertyInstance(
        self.vm, self.name, self, **property_args).to_variable(node)
